//*****************************************************************************
// File Name	: i2cui.c
// 
// Title		: UI -> I2C slave
// Revision		: 1.0
// Notes		: 
// Target MCU	: Atmel AVR series ATtiny2313
// Tool Chain	: WinAVR-20090313
// 
// 
// 
// Revision History:
// When			Who			Description of change
// -----------	-----------	-----------------------
// 10-Sep-2009	S.Takagi	Created the program
//*****************************************************************************

//----- Include Files ---------------------------------------------------------
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include "keyin.h"

#define OFF	0
#define ON	-1

#define CLICK_UNIT 4

#define LED0ON()  (PORTD |=  (1<<PD4))
#define LED0OFF() (PORTD &= ~(1<<PD4))
#define LED1ON()  (PORTD |=  (1<<PD5))
#define LED1OFF() (PORTD &= ~(1<<PD5))
#define LED2ON()  (PORTD |=  (1<<PD6))
#define LED2OFF() (PORTD &= ~(1<<PD6))
#define LED3ON()  (PORTB |=  (1<<PB4))
#define LED3OFF() (PORTB &= ~(1<<PB4))
#define BUZZON()  (PORTB |=  (1<<PB6))
#define BUZZOFF() (PORTB &= ~(1<<PB6))

#include "usiTwiSlave.h"
static const uint8_t TWI_slaveAddress = 0x37;


volatile static uint16_t click_cnt = 0;

enum { S_NOP = 0, S_C, S_l };


#define TIMER1START() (TCCR1B |= (1<<CS10))
#define TIMER1STOP() (TCCR1B &= ~(1<<CS10))

/* Interrupted every 1msec when timer1 is running */
ISR(TIMER1_COMPA_vect)
{
	if(click_cnt==0) {
		TIMER1STOP();
		BUZZOFF();
	} else {
		--click_cnt;
	}

}


void timer1Init(void)
{
	// start timer1
	// Mode 15 : WGM13, WGM12, WGM11, WGM10
	TCCR1B |= (1<<WGM13);
	TCCR1B |= (1<<WGM12);
	TCCR1A |= (1<<WGM11);
	TCCR1A |= (1<<WGM10);

	TCNT1 = 0L;
	OCR1A = 1000L;
	TIMSK |= (1<<OCIE1A);

	sei();
}

void ioInit(void)
{
	DDRD |= (1<<PD4);	// for LED0 output
	DDRD |= (1<<PD5);	// for LED1 output
	DDRD |= (1<<PD6);	// for LED2 output
	DDRB |= (1<<PB4);	// for LED3 output
	DDRB |= (1<<PB6);	// for BUZZER output
	LED0OFF();
	LED1OFF();
	LED2OFF();
	LED3OFF();
	BUZZOFF();
}

uint8_t ioGetPort(void)
{
	return ( ((PINB & 0x50)>>1) | ((PIND & 0xF0)>>4) );
}


int main()
{
	// variables
	//	static uint8_t semaphore = 0xFF; // busy
	static uint8_t state = S_NOP;
	static uint8_t temp, dat;

	// set I/O
	ioInit();

	// I2C Slave initialization
	usiTwiSlaveInit( TWI_slaveAddress );

	// initial value
	keyInit();
	reInit();
	timer1Init();

	// main loop
	while(1) {

		sleep_mode();

		if( usiTwiDataInReceiveBuffer() ) {
	        temp = usiTwiReceiveByte();

			switch (state) {
			case S_NOP:
				switch (temp) {
				case 'k':
				case 'K':
					dat = keyGetCmd();
					usiTwiTransmitByte(dat);
					break;
				case 's':
				case 'S':
					dat = keyGetStat();
					usiTwiTransmitByte(dat);
					break;
				case 'e':
				case 'E':
					dat = reGetCmd();
					usiTwiTransmitByte(dat);
					break;
				case 'r': reEnable();  break;
				case 'R': reDisable(); break;

				case 'c':
					click_cnt = CLICK_UNIT;
					BUZZON();
					TIMER1START();
					break;
				case 'C':
					state = S_C;
					break;
				case 'l':
					state = S_l;
					break;
				case 'L':
					dat = ioGetPort();
					usiTwiTransmitByte(dat);
					break;

				case 'x': LED0ON();  break;
				case 'X': LED0OFF(); break;
				case 'y': LED1ON();  break;
				case 'Y': LED1OFF(); break;
				case 'z': LED2ON();  break;
				case 'Z': LED2OFF(); break;
				case 'w': LED3ON();  break;
				case 'W': LED3OFF(); break;
				case 'b': BUZZON();  break;
				case 'B': BUZZOFF(); break;
				}
				break;
			case S_C:
				click_cnt = CLICK_UNIT * (temp + 1);
				BUZZON();
				TIMER1START();
				state = S_NOP;
				break;
			case S_l:
				if(temp & 0x01) LED0ON();
				else			LED0OFF();
				if(temp & 0x02) LED1ON();
				else			LED1OFF();
				if(temp & 0x04) LED2ON();
				else			LED2OFF();
				if(temp & 0x08) LED3ON();
				else			LED3OFF();
				state = S_NOP;
				break;
			}
	    }

	}

	return 0;
}
